'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compiler = require('svelte/compiler');
var Markdown = require('markdown-it');
var prettier = require('prettier');
var Prism = require('prismjs');
var path = require('path');
require('prismjs/components/prism-bash');
require('prism-svelte');
var resolve = require('@rollup/plugin-node-resolve');
var rollupPluginSvelte = require('rollup-plugin-svelte');
var virtual = require('@rollup/plugin-virtual');
var rollupPluginTerser = require('rollup-plugin-terser');
var fs = require('fs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Markdown__default = /*#__PURE__*/_interopDefaultLegacy(Markdown);
var prettier__default = /*#__PURE__*/_interopDefaultLegacy(prettier);
var Prism__default = /*#__PURE__*/_interopDefaultLegacy(Prism);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var resolve__default = /*#__PURE__*/_interopDefaultLegacy(resolve);
var rollupPluginSvelte__default = /*#__PURE__*/_interopDefaultLegacy(rollupPluginSvelte);
var virtual__default = /*#__PURE__*/_interopDefaultLegacy(virtual);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

let md;

function preprocessReadme(opts) {
  if (!md) {
    md = new Markdown__default['default']({
      html: true,
      linkify: true,
      typographer: true,
      highlight(source, lang) {
        if (lang === "svelte") {
          const regex = new RegExp('"' + opts.name + '"', "g");
          const modifiedSource = encodeURI(source.replace(regex, '"' + opts.svelte + '"'));
          const formattedCode = prettier__default['default'].format(source, {
            parser: "svelte",
            // @ts-ignore
            svelteBracketNewLine: true,
          });
          const svelteCode = Prism__default['default'].highlight(formattedCode, Prism__default['default'].languages.svelte, "svelte");
          return `<pre class="language-${lang}" data-svelte="${modifiedSource}">{@html \`${svelteCode}\`}</pre>`;
        }

        try {
          return `<pre class="language-${lang}">{@html \`${Prism__default['default'].highlight(
            source,
            Prism__default['default'].languages[lang],
            lang
          )}\`}</pre>`;
        } catch (e) {
          console.error(`Could not highlight language "${lang}".`);
          return "";
        }
      },
    });
  }

  return {
    markup: ({ content, filename }) => {
      if (/node_modules/.test(filename) || !filename.endsWith(".md")) return null;
      let script_content = "";
      let style_content = "";
      let result = md.render(content);
      let cursor = 0;

      const ast = compiler.parse(result);

      compiler.walk(ast, {
        enter(node, parent) {
          if (opts.prefixUrl && node.type === "Attribute" && node.name === "href") {
            const value = node.value[0];

            if (value && !value.raw.startsWith("/")) {
              const relative_path = path__default['default'].join(opts.prefixUrl, value.raw);
              result = result.replace(value.raw, relative_path);
              cursor += (relative_path - value.raw).length;
            }
          }

          if (node.type === "Style") {
            style_content += result.slice(node.content.start, node.content.end);
            const replace_style = result.slice(node.start + cursor, node.end + cursor);
            result = result.replace(replace_style, "");
            cursor -= replace_style.length;
          }

          if (node.type === "Attribute" && node.name === "data-svelte") {
            const raw_value = node.value[0].raw;
            const value = decodeURI(raw_value);
            const value_ast = compiler.parse(value);
            const markup =
              `<div class="code-fence">` + value.slice(value_ast.html.start, value_ast.html.end) + "</div>";
            const replace = result.slice(parent.start + cursor, parent.end + cursor);
            result = result.replace(replace, markup + replace.replace(raw_value, ""));
            cursor += markup.length - raw_value.length;

            compiler.walk(value_ast, {
              enter(node) {
                if (node.type === "Script") {
                  script_content += value.slice(node.content.start, node.content.end);
                }
              },
            });
          }
        },
      });

      return {
        code: `
            <script>${script_content}</script>
            <style>
              :global(.token.language-javascript) { color: #24292e; }
              :global(.token.language-javascript .function) { color: #005cc5; }
              :global(.token.language-javascript .string) { color: #032f62; }
              :global(.token.language-javascript .number) { color: #005cc5; }
              :global(.token.language-javascript .keyword) { color: #d73a49; }
              :global(.token.each) { color: #d73a49; }
              :global(.token.punctuation) { color: #24292e }
              :global(.token.tag) { color: #22863a; }
              :global(.token.attr-name) { color: #6f42c1; }
              :global(.token.operator) { color: #d73a49; }
              :global(.token.comment) { color: #6a737d; }

              :global(.language-css) { color: #032f62; }
              :global(.language-css .selector) { color: #22863a; }
              :global(.language-css .property) { color: #005cc5; }

              .code-fence { padding: 45px 15px; border: 1px solid #eaecef; border-bottom: 0; }

              main {
                box-sizing: border-box;
                max-width: 980px;
                margin: 0 auto;
                padding: 45px;
              }
        
              @media (max-width: 767px) {
                main { padding: 15px; }
              }

              ${style_content}
            </style>
            <main class="markdown-body">${result}</main>`,
      };
    },
  };
}

function getPackageJSON() {
  try {
    const path_pkg = path__default['default'].join(process.cwd(), "package.json");
    const pkg = JSON.parse(fs__default['default'].readFileSync(path_pkg, "utf-8"));
    if (!pkg.name) throw Error(`Package name is required as "name".`);
    if (!pkg.svelte) throw Error(`Svelte code entry is required as "svelte".`);

    return {
      name: pkg.name,
      svelte: pkg.svelte,
    };
  } catch (error) {
    console.log(error);
    process.exit(1);
  }
}

function createConfig(opts) {
  return {
    watch: { clearScreen: false },
    input: "entry",
    output: {
      format: "iife",
      name: "app",
      file: "public/bundle.js",
      ...(opts.output || {}),
    },
    plugins: [
      virtual__default['default']({
        entry: `import App from "./README.md";
                const app = new App({ target: document.body });
                export default app;`,
      }),
      rollupPluginSvelte__default['default']({
        ...(opts.svelte || {}),
        extensions: [".svelte", ".md"],
        preprocess: [preprocessReadme({ ...getPackageJSON(), prefixUrl: opts.prefixUrl })],
      }),
      resolve__default['default'](),
      opts.minify === true && rollupPluginTerser.terser(),
    ].filter(Boolean),
  };
}

exports.default = createConfig;
exports.preprocessReadme = preprocessReadme;
